"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Webhook = void 0;
const debug_ts_1 = require("../../platform/deps/debug.ts");
const stream_consumers_ts_1 = require("../../vendor/stream-consumers.ts");
const d = (0, debug_ts_1.debug)('telegraf:webhook');
class Webhook {
    opts;
    server;
    constructor(opts) {
        this.opts = opts;
    }
    async *[Symbol.asyncIterator]() {
        d('Starting server');
        this.server = this.opts.tls
            ? Deno.listenTls({ ...this.opts.tls, port: this.opts.port })
            : Deno.listen({ port: this.opts.port });
        for await (const conn of this.server) {
            for await (const e of Deno.serveHttp(conn)) {
                const body = e.request.body;
                if (!body)
                    e.respondWith(new Response());
                // respond somehow
                else
                    yield (await (0, stream_consumers_ts_1.json)(body));
            }
        }
    }
    async start(handleUpdate, handleError) {
        if (this.server)
            throw new Error('Webhook already running. Do not call start() twice!');
        try {
            // fix error handling
            for await (const update of this)
                handleUpdate(update).catch(handleError);
        }
        finally {
            d('Webhook stopped');
            this.stop();
        }
    }
    stop() {
        this.server?.close();
        this.server = undefined;
    }
}
exports.Webhook = Webhook;
